@@ -19,7 +19,7 @@
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <https://www.gnu.org/licenses/>.
 *
-* 20230609 raf <raf@raf.org>
+* 20231013 raf <raf@raf.org>
 */
 
 #define _GNU_SOURCE /* For FNM_EXTMATCH and FNM_CASEFOLD in <fnmatch.h> */
@@ -69,7 +69,8 @@ static void help_message(void)
 	printf("  -V --version - Show the version message, then exit\n");
 	printf("  -N           - Don't read system-wide config (" RAWHIDE_CONF ")\n");
 	printf("  -n           - Don't read user-specific config (~" RAWHIDE_RC ")\n");
-	printf("  -f fname     - Read functions and/or expression from a file/stdin\n");
+	printf("  -f-          - Read functions and/or expression from stdin\n");
+	printf("  -f fname     - Read functions and/or expression from a file\n");
 	printf(" [-e] 'expr'   - Read functions and/or expression from the cmdline\n");
 	printf("\n");
 	printf("traversal options:\n");
@@ -101,6 +102,7 @@ static void help_message(void)
 	printf("  -v           - Verbose: All columns, implies -ldiBsSac (unless -xXU0L)\n");
 	printf("  -0           - Output null chars instead of newlines (for xargs -0)\n");
 	printf("  -L format    - Output matching entries in a user-supplied format\n");
+	printf("  -j           - Output matching entries as JSON (same as -L \"%%j\\n\")\n");
 	printf("\n");
 	printf("path format options:\n");
 	printf("  -Q           - Enclose paths in double quotes\n");
@@ -425,8 +427,8 @@ int main(int argc, char *argv[]);
 Parse the command line:
 
 The --help and --version options are supported (if first).
-The -e, -x, -X, and -L options can only appear once.
-The -l, -0, -L, -x, -X, and -U options are mutually exclusive.
+The -e, -x, -X, -L, and -j options can only appear once.
+The -l, -0, -L, -j, -x, -X, and -U options are mutually exclusive.
 
 Read /etc/rawhide.conf and /etc/rawhide.conf.d (unless -N).
 Read ~/.rhrc and ~/.rhrc.d (unless -n).
@@ -452,6 +454,7 @@ The -l option and other options include more details.
 There are many options to affect the formatting.
 The -0 option outputs nul characters rather than newlines.
 The -L option outputs information in a user-defined format.
+The -j option outputs information as JSON (same as -L "%j\n").
 The -x option executes a shell command for each
 matching file instead.
 The -X option is the same, but executes each command
@@ -470,10 +473,22 @@ int main(int argc, char *argv[])
 	llong max_pathlen, pathbufsize;
 	struct stat statbuf[1];
 	int o, i, expr_index = 0, any = 0, stdin_read = 0;
+	uid_t uid;
+	gid_t gid;
 
 	setlocale(LC_ALL, "");
 	prog_name = argv[0];
 
+	/* Revoke any unexpected privileges */
+
+	if (getegid() != (gid = getgid()))
+		if (setgid(gid) == -1 || getegid() != getgid() || env_flag("RAWHIDE_TEST_SETGID_FAILURE"))
+			fatalsys("failed to revoke unexpected setgid privileges");
+
+	if (geteuid() != (uid = getuid()))
+		if (setuid(gid) == -1 || geteuid() != getuid() || env_flag("RAWHIDE_TEST_SETUID_FAILURE"))
+			fatalsys("failed to revoke unexpected setuid privileges");
+
 	/* Defaults */
 
 	opt_h = 0;                  /* -h (help) */
@@ -508,7 +523,7 @@ int main(int argc, char *argv[])
 	attr.ctime_column = 0;      /* -c */
 	attr.verbose = 0;           /* -v */
 	attr.nul = 0;               /* -0 */
-	attr.format = NULL;         /* -L */
+	attr.format = NULL;         /* -L/-j */
 
 	attr.quote_name = 0;        /* -Q */
 	attr.escape_name = 0;       /* -E/-b */
@@ -562,7 +577,7 @@ int main(int argc, char *argv[])
 	if (argc >= 2 && !strcmp(argv[1], "--version"))
 		version_message();
 
-	while ((o = getopt(argc, argv, ":hVNnf:e:rm:M:D1yYx:X:UldiBsSgoaucv0L:QEbqptFHIT#?:")) != -1)
+	while ((o = getopt(argc, argv, ":hVNnf:e:rm:M:D1yYx:X:UldiBsSgoaucv0L:jQEbqptFHIT#?:")) != -1)
 	{
 		switch (o)
 		{
@@ -814,7 +829,7 @@ int main(int argc, char *argv[])
 					fatal("missing -L option argument (see %s -h for help)", prog_name);
 
 				if (attr.format)
-					fatal("too many -L options");
+					fatal("too many -L or -j options");
 
 				attr.format = optarg;
 				attr.visitf = visitf_format;
@@ -822,6 +837,17 @@ int main(int argc, char *argv[])
 				break;
 			}
 
+			case 'j':
+			{
+				if (attr.format)
+					fatal("too many -L or -j options");
+
+				attr.format = "%j\n";
+				attr.visitf = visitf_format;
+
+				break;
+			}
+
 			case 'Q':
 			{
 				attr.quote_name = 1;
@@ -956,7 +982,7 @@ int main(int argc, char *argv[])
 		fatal("-x and -0 options are mutually exclusive");
 
 	if (attr.command && !attr.local && attr.format)
-		fatal("-x and -L options are mutually exclusive");
+		fatal("-x and -L/-j options are mutually exclusive");
 
 	if (attr.command && attr.local && opt_l)
 		fatal("-X and -l options are mutually exclusive");
@@ -965,13 +991,13 @@ int main(int argc, char *argv[])
 		fatal("-X and -0 options are mutually exclusive");
 
 	if (attr.command && attr.local && attr.format)
-		fatal("-X and -L options are mutually exclusive");
+		fatal("-X and -L/-j options are mutually exclusive");
 
 	if (opt_l && attr.nul)
 		fatal("-l and -0 options are mutually exclusive");
 
 	if (opt_l && attr.format)
-		fatal("-l and -L options are mutually exclusive");
+		fatal("-l and -L/-j options are mutually exclusive");
 
 	if (attr.unlink && attr.command && !attr.local)
 		fatal("-x and -U options are mutually exclusive");
@@ -986,10 +1012,10 @@ int main(int argc, char *argv[])
 		fatal("-0 and -U options are mutually exclusive");
 
 	if (attr.nul && attr.format)
-		fatal("-0 and -L options are mutually exclusive");
+		fatal("-0 and -L/-j options are mutually exclusive");
 
 	if (attr.unlink && attr.format)
-		fatal("-U and -L options are mutually exclusive");
+		fatal("-U and -L/-j options are mutually exclusive");
 
 	if (attr.escape_name && attr.mask_name)
 		fatal("-E/-b and -q options are mutually exclusive");
