@@ -19,7 +19,7 @@
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <https://www.gnu.org/licenses/>.
 *
-* 20230609 raf <raf@raf.org>
+* 20231013 raf <raf@raf.org>
 */
 
 #define _GNU_SOURCE /* For FNM_EXTMATCH and FNM_CASEFOLD in <fnmatch.h> */
@@ -253,6 +253,15 @@ static symbol_t init_syms[] =
 	{ ".reilink",  PATMOD, 0, c_reilink, NULL },
 #endif
 
+	{ ".body",     PATMOD, 0, c_body,    NULL },
+#ifdef FNM_CASEFOLD
+	{ ".ibody",    PATMOD, 0, c_ibody,   NULL },
+#endif
+#ifdef HAVE_PCRE2
+	{ ".rebody",   PATMOD, 0, c_rebody,  NULL },
+	{ ".reibody",  PATMOD, 0, c_reibody, NULL },
+#endif
+
 #ifdef HAVE_MAGIC
 	{ ".what",     PATMOD, 0, c_what,     NULL },
 #ifdef FNM_CASEFOLD
