@@ -19,7 +19,7 @@
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <https://www.gnu.org/licenses/>.
 *
-* 20230609 raf <raf@raf.org>
+* 20231013 raf <raf@raf.org>
 */
 
 #define _GNU_SOURCE /* For FNM_EXTMATCH and FNM_CASEFOLD in <fnmatch.h> and wcswidth() in <wchar.h> */
@@ -173,6 +173,7 @@ static void caches_init(void)
 	attr.linkdirsize_done = 0;
 	attr.what_done = 0;
 	attr.mime_done = 0;
+	attr.body_done = 0;
 	attr.facl_done = 0;
 	attr.facl = NULL;
 	attr.facl_verbose = NULL;
@@ -718,6 +719,13 @@ int rawhide_search(char *fpath)
 		attr.ttybuf = NULL;
 	}
 
+	if (attr.body)
+	{
+		free(attr.body);
+		attr.body = NULL;
+		attr.body_size = 0;
+	}
+
 	wcoffset(NULL);
 
 	return rc;
@@ -1784,9 +1792,6 @@ This is for better security when the -X option or "cmd".sh is used.
 When changing to each matching file's directory, and then executing
 a shell command, we don't want those directories in $PATH.
 
-If rh is setuid or setgid for some silly reason, $PATH is set to
-"/bin:/usr/bin:/sbin:/usr/sbin".
-
 */
 
 int remove_danger_from_path(void)
@@ -1796,13 +1801,6 @@ int remove_danger_from_path(void)
 
 	#define DEFAULT_PATH "/bin:/usr/bin:/sbin:/usr/sbin"
 
-	if (geteuid() != getuid() || getegid() != getgid())
-	{
-		debug_extra(("path not safe, setting default path"));
-
-		return setenv("PATH", DEFAULT_PATH, 1);
-	}
-
 	if (!(oldpath = getenv("PATH")) || !*oldpath)
 	{
 		debug_extra(("no path, setting default path"));
@@ -2298,7 +2296,7 @@ static char *json(void)
 	static char buf[JSON_BUFSIZE];
 	struct passwd *pwd;
 	struct group *grp;
-	char *base, *selinux, *acl, *ea;
+	char *base, *selinux, *ea;
 	int pos = 0;
 
 	pos += ssnprintf(buf + pos, JSON_BUFSIZE - pos, "{");
@@ -2367,12 +2365,12 @@ static char *json(void)
 	pos += ssnprintf(buf + pos, JSON_BUFSIZE - pos, "\"generation\":%lu, ", get_gen());
 	#endif
 
-	if ((acl = get_acl(1)))
+	if (get_acl(1))
 	{
-		pos += add_field(buf + pos, JSON_BUFSIZE - pos, "access_control_list", acl);
+		pos += add_field(buf + pos, JSON_BUFSIZE - pos, "access_control_list", attr.facl);
 
 		if (attr.facl_verbose)
-			pos += add_field(buf + pos, JSON_BUFSIZE - pos, "access_control_list_verbose", acl);
+			pos += add_field(buf + pos, JSON_BUFSIZE - pos, "access_control_list_verbose", attr.facl_verbose);
 	}
 
 	if ((ea = get_ea(1)) && attr.fea_ok)
